from configparser import ConfigParser
from click import echo
from os.path import expanduser

home = expanduser("~")

VERSION = '0.1.1'
ROOT_DIR = '/root'

HOST_ADDRESS = 'eubflow.eubchain.com'
HTTP_ADDRESS = 'https://{0}'.format(HOST_ADDRESS)
WS_ADDRESS = 'wss://{0}:54188'.format(HOST_ADDRESS)
EUBC_SERVER_ADDRESS = 'http://localhost:8088'
USB_HTTP_ADDRESS = 'https://usb.eubchain.com'

EUBC_LOCAL_ADDRESS = 'http://localhost:8088'

ENABLE_VERBOSE = False
DOCKER_WHITE_LIST = ['rancher_agent', 'eubc_docker_white_list']
CONFIG_FILE_LOCATION = '{0}/.eubc_config.ini'.format(home)

DEFAULT_CONFIG = {
    'USB_HTTP_ADDRESS': USB_HTTP_ADDRESS,
    'HOST_ADDRESS': HOST_ADDRESS,
    'HTTP_ADDRESS': HTTP_ADDRESS,
    'WS_ADDRESS': WS_ADDRESS,
    'EUBC_ADDRESS_PRIVATE': None,
    'EUBC_SERVER_ADDRESS': EUBC_SERVER_ADDRESS,
    'API_SECRET': None
}


class Config:
    def __init__(self):
        self.config = ConfigParser()
        self.config.read(CONFIG_FILE_LOCATION)

    def set(self, key, value):
        if key in DEFAULT_CONFIG.keys():
            self.config.set('DEFAULT', key.lower(), value)
            with open(CONFIG_FILE_LOCATION, 'w+') as config_file:
                self.config.write(config_file)
        else:
            echo('Unsupported configuration information')

    def get(self, key):
        return self.config.get('DEFAULT', key.lower(), fallback=DEFAULT_CONFIG[key.upper()])



import os

import click

from .utils import init_file_server_by_key


class Put:
    def __init__(self, key, directory='code', origin_path=None):
        self.key = key
        self.directory = directory
        self.origin_path = origin_path

    def recreate_share(self, file_service):
        file_service.create_share(self.key)

    def recursive_put(self, path, file_service):
        for file_path in os.listdir(path):
            if file_path not in '.eub':
                _file_path = os.path.join(path, file_path)

                if os.path.isdir(_file_path):
                    self.recursive_put(_file_path, file_service)
                else:
                    self.put_file(_file_path, file_service)

    def put(self, path, is_directory=False):
        if os.path.isdir(path) and not is_directory:
            click.echo("please add -f or --folder parameters to upload folder.")
        else:

            self.origin_path = path

            file_service = init_file_server_by_key(self.key)

            self.recreate_share(file_service)

            file_service.create_directory(self.key, self.directory)

            if is_directory:
                self.recursive_put(path, file_service)
            else:
                self.put_file(path, file_service)

    def put_file(self, file_path, file_service):
        sub_dir = os.path.dirname(os.path.relpath(file_path, self.origin_path))
        upload_directory = self.directory
        if sub_dir:
            upload_directory = "%s/%s" % (upload_directory, sub_dir)
            directories = upload_directory.split('/')
            temp_directory = ''
            for directory in directories:
                temp_directory += directory + '/'
                file_service.create_directory(self.key, temp_directory)

        upload_result = file_service.create_file_from_path(self.key, upload_directory, os.path.basename(file_path),
                                                           file_path)
        if upload_result is None:
            click.echo("%s upload success ." % os.path.basename(file_path))
        else:
            click.echo("%s upload fail ." % os.path.basename(file_path))



# coding=utf-8
import json
import functools
import urllib
import websocket
import argparse
import tabulate
import requests
from cmd2 import Cmd, with_argparser
from .config import Config

# Format to use with tabulate module when displaying tables
TABLE_FORMAT = 'grid'

# Create a function to format a fixed-width table for pretty-printing using the desired table format
table = functools.partial(tabulate.tabulate, tablefmt=TABLE_FORMAT)


class ProjectClient:
    def __init__(self, key, task_id, need_cal):
        self.need_calc = need_cal
        self.key = key
        self.task_id = task_id
        self.config = Config()

    def start(self):
        project_client_self = self

        def get_task_machines():
            return requests.post(
                url='{0}/project/project/machines'.format(project_client_self.config.get('HTTP_ADDRESS')),
                json={'task_id': project_client_self.task_id},
                headers={'Content-Type': 'application/json'}).json()

        class CmdLineApp(Cmd):
            debug = True
            prompt = 'Eub Cli [>]'
            intro = 'Welcome to Eub Cli !'

            def __init__(self):
                # Set use_ipython to True to enable the "ipy" command which embeds and interactive
                # IPython shell
                Cmd.__init__(self, use_ipython=False)
                del Cmd.do_edit, Cmd.do_alias, Cmd.do_load, Cmd.do_py, \
                    Cmd.do_shell, Cmd.do_unalias, Cmd.do_pyscript, Cmd.do_set, Cmd.do_shortcuts

            azure_parser = argparse.ArgumentParser()
            azure_parser.add_argument('-d', '--download', action='store_true',
                                      help='Download resources to task machine')
            azure_parser.add_argument('id', help='Machine id')

            @with_argparser(azure_parser)
            def do_azure(self, args):
                azure_self = self
                if args.id and args.download:
                    def on_server_message(server_ws, message):
                        info = json.loads(message)
                        if info.get('type') == 'runtime_log':
                            azure_self.poutput(info.get('data'))

                    def on_server_onopen(server_ws):
                        azure_self.poutput('Azure start')

                    def on_server_error(server_ws, err):
                        azure_self.poutput('Server connection error:{0}'.format(err))

                    def on_server_close(server_ws):
                        azure_self.poutput('Server connection close')

                    server_ws = websocket.WebSocketApp(
                        '{0}?type=azure_download&task_id={1}&unique_id={2}&key='.format(
                            project_client_self.config.get('WS_ADDRESS'), project_client_self.task_id, args.id,
                            project_client_self.key
                        ),
                        on_message=on_server_message,
                        on_error=on_server_error,
                        on_close=on_server_close)
                    try:
                        server_ws.on_open = on_server_onopen
                        server_ws.run_forever()
                    except (KeyboardInterrupt,):
                        server_ws.close()

            logs_parser = argparse.ArgumentParser()
            logs_parser.add_argument('id', help='Machine id')

            @with_argparser(logs_parser)
            def do_logs(self, args):
                logs_self = self
                if args.id:
                    def on_server_message(server_ws, message):
                        info = json.loads(message)
                        if info.get('type') == 'runtime_log':
                            logs_self.poutput(info.get('data'))

                    def on_server_onopen(server_ws):
                        logs_self.poutput('Logs start')

                    def on_server_error(server_ws, err):
                        logs_self.poutput('Server connection error:{0}'.format(err))

                    def on_server_close(server_ws):
                        logs_self.poutput('Server connection close')

                    server_ws = websocket.WebSocketApp(
                        '{0}?type=logs&task_id={1}&unique_id={2}'.format(
                            project_client_self.config.get('WS_ADDRESS'),
                            project_client_self.task_id,
                            args.id
                        ),
                        on_message=on_server_message,
                        on_error=on_server_error,
                        on_close=on_server_close)
                    try:
                        server_ws.on_open = on_server_onopen
                        server_ws.run_forever()
                    except (KeyboardInterrupt,):
                        server_ws.close()

            run_parser = argparse.ArgumentParser()
            run_parser.add_argument('id', type=str, help='Machine id')
            run_parser.add_argument('cmd', type=str, help='Run command line')

            @with_argparser(run_parser)
            def do_run(self, args):
                run_self = self

                def on_server_message(server_ws, message):
                    info = json.loads(message)
                    if info.get('type') == 'runtime_log':
                        run_self.poutput(info.get('data'))

                def on_server_onopen(server_ws):
                    run_self.poutput('Task start')

                def on_server_error(server_ws, err):
                    run_self.poutput('Server connection error:{0}'.format(err))

                def on_server_close(server_ws):
                    run_self.poutput('Server connection close')

                request_parameters = {
                    'type': 'run',
                    'task_id': project_client_self.task_id,
                    'unique_id': args.id,
                    'image': 'tensorflow/tensorflow:1.7.0-gpu',
                    'cmd': args.cmd
                }
                server_ws = websocket.WebSocketApp(
                    '{0}?{1}'.format(project_client_self.config.get('WS_ADDRESS'),
                                     urllib.urlencode(request_parameters)),
                    on_message=on_server_message,
                    on_error=on_server_error,
                    on_close=on_server_close)
                try:
                    server_ws.on_open = on_server_onopen
                    server_ws.run_forever()
                except (KeyboardInterrupt,):
                    server_ws.close()

            machine_parser = argparse.ArgumentParser()

            @with_argparser(machine_parser)
            def do_machine(self, args):
                """Repeats what you tell me to."""
                machines = get_task_machines()
                data = machines.get('data')
                table_header = ['Id', 'HostName', 'PublicIp', 'IpAddress', 'Gpu', 'benchmark']
                table_data = []
                for machine in data:
                    gpu_str = ''
                    for card in machine.get('gpu'):
                        gpu_str += '{0}({1})\r\n'.format(card.get('Name'), card.get('Memory'))
                    table_data.append([
                        machine.get('unique_id'),
                        machine.get('host_name'),
                        machine.get('public_ip'),
                        machine.get('ip_address'),
                        gpu_str,
                        machine.get('benchmark')
                    ])
                self.ptable(table_data, table_header)

            def ptable(self, tabular_data, headers=()):
                """Format tabular data for pretty-printing as a fixed-width table and then display it using a pager.

                :param tabular_data: required argument - can be a list-of-lists (or another iterable of iterables), a list of
                                     named tuples, a dictionary of iterables, an iterable of dictionaries, a two-dimensional
                                     NumPy array, NumPy record array, or a Pandas dataframe.
                :param headers: (optional) - to print nice column headers, supply this argument:
                                - headers can be an explicit list of column headers
                                - if `headers="firstrow"`, then the first row of data is used
                                - if `headers="keys"`, then dictionary keys or column indices are used
                                - Otherwise, a headerless table is produced
                """
                formatted_table = table(tabular_data, headers=headers)
                self.ppaged(formatted_table)

            def do_quit(self, _):
                """Exits this application."""
                ws.close()
                self._should_quit = True
                return self._STOP_AND_EXIT

        def on_message(ws, message):
            info = json.loads(message)
            if info.get('type') == 'device_login':
                print 'New device Login in'
                print 'Current Device Benchmark:'
                current_register = info.get('currentRegister')
                total_benchmark = 0
                for register in current_register:
                    data = register.get('data')
                    if data:
                        print '{0} ({1})'.format(data.get('unique_id'), data.get('benchmark'))

                        total_benchmark += data.get('benchmark')
                        if total_benchmark >= self.need_calc:
                            c = CmdLineApp()
                            c.cmdloop()

            elif info.get('type') == 'runtime_log':
                print info.get('data')

        def on_error(ws, error):
            print 2, error

        def on_close(ws):
            print 3

        def on_open(ws):
            print 'Connection server success'

        ws = websocket.WebSocketApp('{0}?type=task_requester&task_id={1}'.format(project_client_self.config.get(
            'WS_ADDRESS'), project_client_self.task_id),
            on_message=on_message,
            on_error=on_error,
            on_close=on_close)
        ws.on_open = on_open
        ws.run_forever()



import platform
import random
import socket
import string
import threading
from time import sleep

import websocket
import sseclient
import json
import docker
from ..utils import get_host_ip, get_public_ip
from ..api import get_eubserver_event
from ..eubc_server_api import EubcServerApi
from ..config import Config
from ..get import Get
from ..put import Put
from click import echo
from multiprocessing import Process

do_acceptor_event_process = []


def clean_docker():
    print 'Start stop containers'
    docker_client = docker.from_env()
    containers = docker_client.containers.list()
    for container in containers:
        container.stop()
    print 'Stop containers'
    return docker_client


def on_acceptor_message(ws, message, data):
    """Send Runtime logs"""

    def send_runtime_log(messages):
        ws.send(json.dumps({
            'type': 'runtime_log',
            'data': messages,
            'task_id': data.get('id'),
            'unique_id': unique_id
        }))

    def do_running():
        print 'Image:{0},Cmd:{1}'.format(info.get('image'), info.get('cmd'))
        key = info.get('key')
        if info.get('pull_code') and info.get('key') == 'true':
            print 'Start pull code'
            Get(info.get('key')).get('/root', True)
            print 'Pull code success'
        docker_client = clean_docker()

        container = docker_client.containers.run(
            info.get('image'),
            info.get('cmd'),
            detach=True,
            auto_remove=True,
            runtime='nvidia',
            volumes={
                '/root': {'bind': '/notebooks', 'mode': 'rw'}
            }
        )
        container_id = container.attrs['Id']

        print 'Current container Id:{0}'.format(container_id)
        send_runtime_log(container_id)

        print 'Start transfer logs stream'
        for line in container.logs(stream=True):
            send_runtime_log(line.strip())

        Put(key, 'result').put('/root/result', True)

    def clean_running_process():
        """Terminate old run process"""
        for item in do_acceptor_event_process:
            if item.get('name') == 'run' and isinstance(item.get('process'), Process):
                item.get('process').terminate()

    info = json.loads(message)
    if info.get('type') == 'runtime_log':
        print info.get('data')
    elif info.get('type') == 'stop':
        send_runtime_log('Start stop containers')
        clean_running_process()
        clean_docker()
        send_runtime_log('Stop containers success')
    elif info.get('type') == 'run':
        clean_running_process()
        """Start running process"""
        p = Process(target=do_running, )
        do_acceptor_event_process.append({
            'name': 'run',
            'process': p
        })
        p.start()
    elif info.get('type') == 'upload-result':
        send_runtime_log('Start upload result')
        key = info.get('key')
        Put(key, 'result').put('/root/result', True)
        send_runtime_log('Upload result success')


class SSE:
    def __init__(self, address):
        self.address = address
        self.eubc_server_api = EubcServerApi()
        self.config = Config()

    def start(self):
        can_receive_task = True
        lock_can_task = False
        client = sseclient.SSEClient(get_eubserver_event())
        current_task_id = None

        for event in client.events():
            event_name = event.event
            data = json.loads(event.data)
            task_info = self.eubc_server_api.task_info(data.get('taskId'))
            echo('Task type:{0},Task id:{1}'.format(event_name, data.get('taskId')))

            if task_info.get('status') == 'success':
                data = task_info.get('data')
                info_json = json.loads(data.get('infoJson'))

                if can_receive_task:
                    if event_name == 'request':
                        if info_json.get('type').lower() == 'eubflow':
                            echo('TensorFlow taskId:{0}'.format(data.get('id')))
                            task_accept_response = self.eubc_server_api.task_accept(data.get('id'))
                            if task_accept_response.get('status') == 'error':
                                echo('Accept taskId:{0} request fail'.format(task_accept_response.get('message')))
                            elif task_accept_response.get('status') == 'success':
                                echo('Accept taskId:{0} request success'.format(data.get('id')))

                    elif event_name == 'accept':
                        if data.get('acceptor') == self.address:
                            can_receive_task = False
                            current_task_id = data.get('id')

                            def do_acceptor_event():
                                timout_seconds = int(data.get('timeoutSeconds'))
                                echo('Receive the task successfully and begin execution, taskId:{0}'.format(
                                    current_task_id))

                                def on_message(ws, message):
                                    on_acceptor_message(ws, message, data)

                                    # elif info.get('type') == 'azure_download':
                                    #     send_runtime_log('Start download azure resources')
                                    #     send_runtime_log('Download azure complete')

                                def on_error(ws, error):
                                    print 2, error

                                def on_close(ws):
                                    print 3

                                def on_open(ws):
                                    gpus = []
                                    global unique_id
                                    if 'ubuntu' in platform.platform().lower():
                                        from subprocess import call
                                        # unique_id = call("dmidecode -t 4 | grep ID | sed 's/.*ID://;s/ //g'")
                                        # print unique_id
                                        unique_id = ''.join(random.sample(string.ascii_letters + string.digits, 8))
                                        print unique_id
                                        from gpuinfo.nvidia import get_gpus
                                        try:
                                            for gpu in get_gpus():
                                                gpus.append({
                                                    "Name": gpu.name,
                                                    "Memory": '{0} GB'.format(round(gpu.total_memory / 1024, 1) + 1)
                                                })
                                        except:
                                            gpus = []
                                    else:
                                        unique_id = ''.join(random.sample(string.ascii_letters + string.digits, 8))
                                    ws.send(json.dumps({
                                        'type': 'register',
                                        'device_info': {
                                            "hostname": socket.gethostname(),
                                            "ip_address": get_host_ip(),
                                            "gpu": gpus,
                                            "public_ip": get_public_ip(),
                                            'unique_id': unique_id,
                                            'benchmark': 10
                                        },
                                        'task_id': data.get('id')
                                    }))

                                ws = websocket.WebSocketApp(self.config.get('WS_ADDRESS'),
                                                            on_message=on_message,
                                                            on_error=on_error,
                                                            on_close=on_close)
                                ws.on_open = on_open

                                ws_threading = threading.Thread(target=ws.run_forever)
                                ws_threading.start()

                                sleep(timout_seconds)
                                ws.close()
                                can_receive_task = True

                            p = Process(target=do_acceptor_event, )
                            do_acceptor_event_process.append({
                                'name': 'accept',
                                'process': p
                            })
                            p.start()

                if event_name == 'cancel':
                    if not lock_can_task and data.get('id') == current_task_id:
                        lock_can_task = True
                        print 'Task cancel', data
                        """Terminate all process"""
                        for item in do_acceptor_event_process:
                            if isinstance(item.get('process'), Process):
                                item.get('process').terminate()
                        """Clean docker"""
                        clean_docker()
                        """Put result to azure"""
                        Put(info_json.get('project_key'), 'result').put('/root/result', True)
                        lock_can_task = False
                        can_receive_task = True



from json import dumps

import docker
from ws4py.client.threadedclient import WebSocketClient
from ..config import Config
from click import echo


class ContainerLog(WebSocketClient):

    def __init__(self, pivot, protocols=None, extensions=None, heartbeat_freq=None, ssl_options=None, headers=None,
                 exclude_headers=None):
        config = Config()
        super(ContainerLog, self).__init__(config.get('WS_ADDRESS'), protocols, extensions, heartbeat_freq, ssl_options,
                                           headers,
                                           exclude_headers)
        self.pivot = pivot
        self.client = docker.from_env()

    def opened(self):
        echo('WebSocket opened')
        try:
            if len(self.client.containers.list()) > 0:
                echo('Docker client containers more than 1')
                for container in self.client.containers.list():
                    for line in container.logs(stream=True):
                        self.send(dumps({
                            'type': 'eubh',
                            'project_machine_id': self.pivot.get('id'),
                            'data': line
                        }))
            else:
                echo('Docker client containers less 1')
                self.close()
        except:
            self.close()
            self.connect()
            echo('Web socket is close')



from os import system

from eubh import *
import click

_global_options = [
    click.option('--verbose', '-v', 'verbosity', flag_value=2, default=1, help='Enable verbose output'),
]


def global_options(func):
    for option in reversed(_global_options):
        func = option(func)
    return func


@click.group()
def config_group():
    pass


@config_group.command()
@click.argument('key', required=True)
@click.argument('value', required=False)
def config(key, value):
    current_config = Config()
    if value:
        current_config.set(key, value)
    else:
        echo(current_config.get(key))


@click.group()
def get_group():
    pass


@get_group.command()
@click.argument('key', required=True)
@click.argument('path', required=True)
@click.option('--code/--no-code', default=True, help='Get code source')
@click.option('--result/--no-result', default=True, help='Get result')
@click.option('--unzip/--no-unzip', default=False, help='Unzip input')
def get(key, path, code, result, unzip):
    if code:
        Get(key).get(path, unzip)
    if result:
        Get(key, 'result').get(path, unzip)


@click.group()
def put_group():
    pass


@put_group.command()
@click.argument('key', required=True)
@click.argument('path', required=True)
@click.option('--folder/--no-folder', default=True, help='Upload directory')
@click.option('--code/--no-code', default=False, help='Upload code')
@click.option('--result/--no-result', default=False, help='Put result')
def put(key, path, folder, code, result):
    if code:
        Put(key).put(path, folder)
    if result:
        Put(key, 'result').put(path, folder)


@click.group()
def watch_group():
    pass


@watch_group.command()
@global_options
@click.option('-k', '--key')
@click.option('-t', '--time', default=10, help='Loop time (s)')
@click.option('-v', '--verbosity', default=True, help='Verbose mode')
def watch(key, time, verbosity):
    Watch(key, time, verbosity).watch()


@click.group()
def init_group():
    pass


@init_group.command()
def init():
    from eubh.init import init as project_init
    project_init()


@click.group()
def upgrade_group():
    pass


@upgrade_group.command()
def upgrade():
    system('pip install --upgrade eubh --no-cache-dir')


@click.group()
def user_group():
    pass


@user_group.command()
def login():
    from eubh.user import login as user_login
    user_login()


@user_group.command()
@click.argument('private_key', required=True)
def import_private_key(private_key):
    from eubh.user import import_private_key
    import_private_key(private_key)


@click.group()
def version_group():
    pass


@version_group.command()
def version():
    echo(VERSION)


@click.group()
def container_group():
    pass


cli = click.CommandCollection(
    sources=[
        get_group,
        put_group,
        watch_group,
        init_group,
        upgrade_group,
        version_group,
        config_group,
        user_group
    ])


def main():
    cli()


if __name__ == "__main__":
    cli()



import multiprocessing
import platform
import random
import string
from uuid import getnode as get_mac

import json
import tqdm
import docker
import sseclient
import websocket
import threading
from apscheduler.schedulers.background import BackgroundScheduler
from click import echo
from time import sleep

from websocket._exceptions import WebSocketException

from .server import SSE
from .get import *
from .put import *
from .status import *
from .utils import *
from .config import ROOT_DIR, DOCKER_WHITE_LIST, Config
from .logs.container_log import ContainerLog as WriteContainerLog
from .wesockets.container_log import ContainerLog as PushContainerLog
from multiprocessing import Process
from .api import get_chain_sync_progress, get_eubserver_event
from .eubc_server_api import EubcServerApi


def clean_file():
    echo('Remove %s/code and %s/result' % (ROOT_DIR, ROOT_DIR))
    os.popen('rm -rf %s/code %s/result' % (ROOT_DIR, ROOT_DIR))
    echo('Remove files successful')


def create_directories():
    os.makedirs('%s/code' % ROOT_DIR)
    os.makedirs('%s/result' % ROOT_DIR)


def job_write_log(data, verbosity):
    WriteContainerLog(pivot=data, verbosity=verbosity)


def job_real_time_push_log(data, reply=10):
    push_container_log = PushContainerLog(data)
    is_success = False

    while reply > 1 and not is_success:
        try:
            push_container_log.connect()
            is_success = True
        except WebSocketException or KeyboardInterrupt:
            push_container_log.close()
            push_container_log = False
            reply -= 1
            echo('WebSocket connect reply %d' % reply)
    if not is_success:
        echo('WebSocket connect fail')


class Watch:
    def __init__(self, key, time, verbosity):
        config = Config()
        self.config = config
        self.time = time
        self.key = key
        self.api = Api()
        self.host = config.get('HOST_ADDRESS')
        self.webscoket = config.get('WS_ADDRESS')
        echo('Api server:{0} ,Webscoket:{1}'.format(config.get('HTTP_ADDRESS'), config.get('WS_ADDRESS')))
        self.docker_client = docker.from_env()
        self.can_init = True
        self.cache_log = []
        self.jobs = []
        self.is_verbose = True if verbosity else False
        self.eubc_server_api = EubcServerApi()

    def clean_docker(self):
        client = self.docker_client
        for container in client.containers.list():
            if container.attrs['Name'] not in DOCKER_WHITE_LIST:
                container.stop()

    def run_cmd_and_upload_result(self, pivot):
        cmd = pivot.get('cmd')
        if not is_empty(cmd):
            cmd_result_out_put = os.popen(cmd)
            cmd_result_out_put_string = cmd_result_out_put.read()
            self.api.project_machine_update_cmd({"id": pivot.get('id'), "result": cmd_result_out_put_string})
            cmd_result_out_put.close()

    def upload_machine_information(self):
        client = self.docker_client
        current_status = Status.IDLE.value

        client_containers_count = len(client.containers.list())
        if client_containers_count == 0:
            if not is_empty(self.key):
                current_status = Status.COMPLETE.value
        else:
            current_status = Status.IDLE.value

        if client_containers_count > 0:
            if is_empty(self.key):
                self.clean_docker()
            current_status = Status.RUNNING.value

        data = get_device_info()
        data['status_info'] = {
            'key': self.key,
            'status': current_status
        }
        data['mac'] = "machine_%s" % get_mac()
        response = self.api.upload_machine_and_get_task(data)

        if type(response) is not list:
            option = response.get('option')
            project = response.get('project')
            pivot = response.get('pivot')

            key = None
            if project is not None:
                key = project.get('key')

            if not is_empty(key):
                if option == 'init' and self.can_init:
                    self.can_init = False
                    self.clean_current_jobs()
                    clean_file()
                    create_directories()
                    echo('Pull code')
                    Get(key).get(ROOT_DIR, True)
                    echo('Pull code success')
                    self.key = key
                    echo('Creating container.')
                    self.run_cmd_and_upload_result(pivot)
                    echo('Created container.')

                    echo('Creating write log job')
                    write_log = multiprocessing.Process(target=job_write_log,
                                                        args=(pivot, self.is_verbose))
                    write_log.start()
                    echo('Creating real time log')
                    push_log = multiprocessing.Process(target=job_real_time_push_log,
                                                       args=(pivot,))
                    push_log.start()
                    echo('Created real time log')
                    self.jobs.append(write_log)
                    self.jobs.append(push_log)

                elif option == 'cmd':
                    self.run_cmd_and_upload_result(pivot)

            if option == 'clean':
                echo('Cleaning docker')
                self.clean_docker()
                echo('Cleaned docker')
                echo('Cleaning jobs')
                self.clean_current_jobs()
                echo('Cleaned jobs')
                if not is_empty(self.key):
                    echo('Pushing result')
                    Put(self.key, 'result').put('%s/result' % ROOT_DIR, True)
                    echo('Pushed result')
                    self.key = ''
                    self.can_init = True

    def log_is_exist(self, container_id, log):
        cache_logs = self.cache_log
        is_exist_log = False
        for i in cache_logs:
            if cache_logs[i].container_id == container_id and cache_logs[i].log == log:
                is_exist_log = True
                break
        if not is_exist_log:
            self.cache_log.append({
                container_id: container_id,
                log: log
            })
        return is_exist_log

    def watch_upload_docker_log(self):
        client = self.docker_client

        for container in client.containers.list():
            log = container.logs()

            self.api.post_project_container_logs({
                'container_id': container.attrs['Id'],
                'log': log
            })

    def clean_current_jobs(self):
        if len(self.jobs) > 0:
            for job in self.jobs:
                job.terminate()

    def watch(self):
        """start geth ,eubserver in background"""
        jobs = []
        try:
            jobs = download_and_run_eubchain_eubserver()
        except (KeyboardInterrupt, SystemExit):
            for job in jobs:
                if isinstance(job, Process):
                    job.terminate()
        sync_is_done = 0
        with tqdm.tqdm(total=100) as pbar:
            temp_position = 0
            while sync_is_done <= 5:
                try:
                    sync_progress_data = get_chain_sync_progress()
                    current_sync_progress = sync_progress_data.get('data')
                    if current_sync_progress == 100:
                        sync_is_done += 1
                    else:
                        pbar.update(current_sync_progress - temp_position)
                        temp_position = current_sync_progress
                except Exception:
                    echo('Connection eubserver fail, Reply')
                sleep(3)
        echo('Sync Done,wait task')
        echo('Start import privateKey')
        import_private_key_response = self.eubc_server_api.import_private_key()
        echo('Import privateKey success')

        echo('Start sse')
        sse = SSE(import_private_key_response.get('address'))

        sse_thread = threading.Thread(target=sse.start())
        sse_thread.start()
        sse_thread.join()

        # while True:
        #     self.upload_machine_information()
        #     sleep(self.time)
        # scheduler = BlockingScheduler()
        # scheduler.add_job(self.upload_machine_information, 'interval', seconds=self.time, max_instances=1)
        # try:
        #     scheduler.start()
        # except (KeyboardInterrupt, SystemExit):
        #     scheduler.shutdown()



from .api import *
from .config import *
from .get import *
from .init import *
from .put import *
from .status import *
from .utils import *
from .watch import *
from .eubc_server_api import *
from .user import *



import json

import sseclient

from ..api import get_eubserver_event

from .sse import SSE


class ClientSSE(SSE):
    def __init__(self, address):
        SSE.__init__(self, address)
        self.client = sseclient.SSEClient(get_eubserver_event())

    def start(self):
        for event in self.client.events():
            event_name = event.event
            data = json.loads(event.data)
            yield event_name, data

    def close(self):
        self.client.close()

    def return_if_request_is_self(self):
        for event_name, data in self.start():
            if event_name == 'request' and data.get('requester') == self.address:
                self.close()
                return data.get('taskId')



# coding=utf-8
import webbrowser
from .config import Config
from click import echo
from .eubc_server_api import EubcServerApi


def login():
    webbrowser.open('http://eubflow.eubchain.local/user/admin')
    api_secret = raw_input('输入Api Secret:')
    config = Config()
    config.set('API_SECRET', api_secret)
    echo('Login success')


def import_private_key(private_key):
    api = EubcServerApi()
    api.import_private_key(private_key)
    echo('Import privateKey success')



# coding=utf-8
import os
import socket
import zipfile
from ConfigParser import NoOptionError
from json import load
from netifaces import interfaces, ifaddresses, AF_INET
from urllib2 import urlopen
import click
import rarfile
from azure.storage.file import FileService
import hashlib
import multiprocessing
import os
import sys
import psutil
import logging
from .api import Api
from wget import download
from zipfile import ZipFile
from os import path, makedirs, popen
from .config import ROOT_DIR, Config


def restart_program():
    """Restarts the current program, with file objects and descriptors
       cleanup
    """
    try:
        p = psutil.Process(os.getpid())
        for handler in p.get_open_files() + p.connections():
            os.close(handler.fd)
    except Exception, e:
        logging.error(e)

    python = sys.executable
    os.execl(python, python, *sys.argv)


def check_api_secret_is_correct():
    config = Config()
    if not config.get('api_secret'):
        click.echo('Please login before run this command')
        exit(0)
    return True


def un_zip(filename, to_path):
    """unzip zip file"""
    zip_file = zipfile.ZipFile(filename)
    if os.path.exists(to_path):
        os.remove(to_path)
    else:
        os.makedirs(to_path)
    zip_file.extractall(to_path)
    zip_file.close()


def un_rar(file_name, to_path):
    """unrar rar file"""
    rar = rarfile.RarFile(file_name)
    if os.path.exists(to_path):
        os.remove(file_name)
    else:
        os.makedirs(to_path)
    rar.extractall(to_path)
    rar.close()


def init_environment():
    """init ubuntu nvidia docker environment"""
    os.system('curl https://public-packages.oss-cn-beijing.aliyuncs.com/install_nvidia_docker.sh|sh -')


def init_file_server_by_key(key):
    config = Api(key=key).get_config_by_key().get('data')

    file_service = FileService(account_name=config.get('account_name'),
                               account_key=config.get('account_key'),
                               endpoint_suffix=config.get('endpoint_suffix'))

    # logging.basicConfig(format='%(asctime)s %(name)-20s %(levelname)-5s %(message)s', level=logging.INFO)
    # logger = logging.getLogger('azure.storage')
    # handler = logging.StreamHandler()
    # formatter = logging.Formatter('%(asctime)s %(name)-20s %(levelname)-5s %(message)s')
    # handler.setFormatter(formatter)
    # logger.addHandler(handler)
    # logger.setLevel(logging.INFO)

    return file_service


def get_folder_name_from_path(path):
    return os.path.basename(path)


def listdir(path, list_name):
    for file in os.listdir(path):
        file_path = os.path.join(path, file)
        if os.path.isdir(file_path):
            listdir(file_path, list_name)
        else:
            list_name.append(file_path)


def ip4_addresses():
    ip_list = []
    for interface in interfaces():
        for link in ifaddresses(interface).get(AF_INET, ()):
            ip_list.append(link['addr'])
    return ip_list


def get_host_ip():
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    finally:
        s.close()
    return ip


def get_public_ip():
    return load(urlopen('http://jsonip.com'))['ip']


def is_empty(key):
    return key == '' or key is None


def get_device_info():
    import platform
    gpus = []
    if 'ubuntu' in platform.platform().lower():
        from gpuinfo.nvidia import get_gpus
        try:
            for gpu in get_gpus():
                gpus.append({
                    "Name": gpu.name,
                    "Memory": gpu.total_memory
                })
        except:
            click.echo("get gpu information error ")
    return {
        "hostname": socket.gethostname(),
        "ip_address": get_host_ip(),
        "gpu": gpus,
        "public_ip": get_public_ip(),
        'unique_id': os.system("dmidecode -t 4 | grep ID | sed 's/.*ID://;s/ //g'")
    }


def get_bool(prompt):
    while True:
        try:
            return {"true": 'y', "false": 'n'}[input(prompt).lower()]
        except KeyError:
            print "Invalid input please enter True or False!"


def remove_folder(folder):
    os.popen('rm -rf %s' % folder)


def calc_md5(file_path):
    with open(file_path, 'rb') as f:
        md5obj = hashlib.md5()
        md5obj.update(f.read())
        return md5obj.hexdigest()


def download_file(url, to_path, md5=None):
    if os.path.exists(to_path):
        os.unlink(to_path)
    download(url, to_path)
    if md5:
        file_md5 = calc_md5(to_path)
        click.echo('\r\nDownload file md5:{0},Remote file md5:{1}'.format(file_md5, md5))
        if md5 != calc_md5(to_path):
            download_file(url, to_path, md5)


def unzip_file(zip_path, to_path):
    z = ZipFile(zip_path, 'r')
    z.extractall(path=to_path)
    z.close()


def get_ifn_exit(key):
    value = None
    config = Config()
    try:
        value = config.get(key)
    except NoOptionError:
        click.echo('未设置 {0}'.format(key))
    if not value:
        click.echo('未设置 {0}'.format(key))
        exit(0)
    return value


def download_and_run_eubchain_eubserver():
    api = Api()
    data = api.get_eubchain()
    save_path = '{0}/.eub'.format(ROOT_DIR)
    jobs = []
    if not path.exists(save_path):
        makedirs(save_path)
    for item in data:
        download_file_name = '{0}/{1}.zip'.format(save_path, item.get('folder'))
        out_put_folder = '{0}/{1}'.format(save_path, item.get('folder'))

        def t(is_download=True):
            if is_download:
                download_file(item.get('url'), download_file_name, item.get('md5'))
            unzip_file(download_file_name, out_put_folder)
            popen('chmod +x {0}/{1}'.format(out_put_folder, item.get('name')))
            if item.get('name') in 'geth':
                background_process = multiprocessing.Process(target=popen,
                                                             args=(
                                                                 '{0}/{1} --identity "eub-light-client" '
                                                                 '--datadir "{0}/eth" --verbosity 0'.format(
                                                                     out_put_folder, item.get('name')),))
                background_process.start()
                jobs.append(background_process)
            elif item.get('name') in 'eubserver':
                background_process = multiprocessing.Process(target=popen,
                                                             args=('{0}/{1} > /dev/null'.format(out_put_folder,
                                                                                                item.get('name')),))
                background_process.start()
                jobs.append(background_process)

        if not path.exists(out_put_folder):
            makedirs(out_put_folder)
        if not path.exists(download_file_name):
            t()
        else:
            if calc_md5(download_file_name) != item.get('md5'):
                t()
            else:
                t(False)
    return jobs



from .config import Config
from requests import get, post, put
from jwt import encode
from click import echo


class EubcServerApi:
    def __init__(self):
        config = Config()
        self.config = config
        self.eubc_server_url = config.get('EUBC_SERVER_ADDRESS')

    def get_eubc_server_url(self, url):
        return '{0}{1}'.format(self.eubc_server_url, url)

    def get_eubc_account(self):
        return get(self.get_eubc_server_url('/account')).json()

    def generate_secret_data(self, url, data={}):
        return encode(data, self.get_eubc_account().get('address') + '/' + url)

    def create_eubc_account(self, url='/account/create'):
        return post(self.get_eubc_server_url(url), data={'data': self.generate_secret_data(url)}).json()

    def task_info(self, task_id):
        url = '/tasks/{0}'.format(task_id)
        return get(self.get_eubc_server_url(url)).json()

    def task_accept(self, task_id):
        url = '/tasks/{0}/accept'.format(task_id)
        return put(self.get_eubc_server_url(url), data={'data': self.generate_secret_data(url)}).json()

    def task_request(self, data, url='/tasks/request'):
        return post(self.get_eubc_server_url(url),
                    json={'data': self.generate_secret_data(url, data)}).json()

    def import_private_key(self, address_private_key=None, url='/account/importPrivateKey'):
        if not address_private_key:
            address_private_key = self.config.get('EUBC_ADDRESS_PRIVATE')
        if address_private_key:
            return post(self.get_eubc_server_url(url),
                        data={'data': self.generate_secret_data(url, {'privateKey': address_private_key})}).json()
        else:
            echo('Creating new privateKey')
            account = self.create_eubc_account()
            echo('Eubc address created ,address :{0},privateKey:{1}'.format(account.get('address'),
                                                                            account.get('privateKey')))
            self.config.set('EUBC_ADDRESS_PRIVATE', account.get('privateKey'))
            self.import_private_key()



# coding=utf-8
import os

import pick
import click
from configparser import ConfigParser
from .api import *
from .utils import get_ifn_exit, get_bool
from .eubc_server_api import EubcServerApi
from .put import Put
from .server import ClientSSE
from .project_client import ProjectClient

current_config_folder = '{0}/.eub'.format(os.path.abspath('.'))
config_path = '{0}/.config'.format(current_config_folder)


class Project:
    def __init__(self):
        if not os.path.exists(current_config_folder):
            os.makedirs(current_config_folder)
        config = ConfigParser()
        config.read('{0}/.config'.format(current_config_folder))
        self.config = config
        self.api = Api()
        self.eubc_server_api = EubcServerApi()

    def init(self):
        global project_name, project_description, azure_account_name, azure_secret, min_lan, project_type, timeout_seconds, task_reward
        get_ifn_exit('API_SECRET')
        address_private = get_ifn_exit('EUBC_ADDRESS_PRIVATE')
        import_private_key_response = response = self.eubc_server_api.import_private_key(address_private)
        if response.get('status') == 'error':
            click.echo(import_private_key_response.get('message'))
            exit(0)

        project_name = self.config.get('DEFAULT', 'project_name', fallback=None)
        if not project_name:
            project_name = raw_input('请输入项目名称:')
            self.config.set('DEFAULT', 'project_name', project_name)
        project_description = self.config.get('DEFAULT', 'project_description', fallback=None)
        if not project_description:
            project_description = raw_input('请输入项目描述:')
            self.config.set('DEFAULT', 'project_description', project_description)
        azure_account_name = self.config.get('DEFAULT', 'azure_account_name', fallback=None)
        if not azure_account_name:
            azure_account_name = raw_input('请输入Azure账号名称(AccountName):')
            self.config.set('DEFAULT', 'azure_account_name', azure_account_name)
        azure_secret = self.config.get('DEFAULT', 'azure_secret', fallback=None)
        if not azure_secret:
            azure_secret = raw_input('请输入Azure密钥(Secret):')
            self.config.set('DEFAULT', 'azure_secret', azure_secret)
        min_lan = self.config.get('DEFAULT', 'min_lan', fallback=None)
        if not min_lan:
            min_lan = raw_input('请输入需要的闲置资源数:')
            self.config.set('DEFAULT', 'min_lan', min_lan)
        project_type = self.config.get('DEFAULT', 'project_type', fallback=None)
        if not project_type:
            project_type, index = pick.pick(['EubFlow', 'EubRending'], 'Project type:')
            self.config.set('DEFAULT', 'project_type', project_type)
        timeout_seconds = self.config.get('DEFAULT', 'timeout_seconds', fallback=None)
        if not timeout_seconds:
            timeout_seconds = raw_input('项目超时时间(分钟):')
            self.config.set('DEFAULT', 'timeout_seconds', str(int(timeout_seconds) * 60000))
        task_reward = self.config.get('DEFAULT', 'task_reward', fallback=None)
        if not task_reward:
            task_reward = raw_input('项目奖励:')
            self.config.set('DEFAULT', 'task_reward', task_reward)
        key = self.config.get('DEFAULT', 'key', fallback=None)
        if not key:
            response = self.api.project_store({
                'project_name': project_name,
                'project_description': project_description,
                'account_name': azure_account_name,
                'account_key': azure_secret,
                'min_lan': min_lan,
                'product_type': project_type,
                'timeout_seconds': timeout_seconds,
                'task_reward': task_reward
            })
            key = response.get('data').get('key')
            self.config.set('DEFAULT', 'key', key)
        with open(config_path, 'w+') as config_file:
            self.config.write(config_file, 'w+')
        client_sse = ClientSSE(import_private_key_response.get('address'))
        # if get_bool("是否上传代码到Azure?[y/n]"):
        #     click.echo('开始上传Code到Azure')
        #     Put(key).put(os.getcwd(), True)
        response = self.eubc_server_api.task_request({
            'timeoutSeconds': timeout_seconds,
            'infoJson': json.dumps({'project_key': key, 'type': project_type}),
            'taskReward': task_reward
        })
        if response.get('status') == 'error':
            click.echo(response.get('message'))
            exit(0)
        elif response.get('status') == 'success':
            click.echo('发布任务成功,等待设备接入')
            task_id = client_sse.return_if_request_is_self()
            project_client = ProjectClient(key, task_id, 10)
            project_client.start()
        else:
            click.echo('发布任务失败')



from json import dumps

from docker import from_env
from ws4py.client.threadedclient import WebSocketClient

from ..config import ROOT_DIR, Config


def write_docker_log(pivot, verbosity):
    client = from_env()
    container_lists = client.containers.list()
    config = Config()
    if len(container_lists) > 0:
        ws = WebSocketClient(url=config.get('WS_ADDRESS'))
        try:
            ws.connect()
            for container in container_lists:
                for line in container.logs(stream=True):
                    with open('%s/result/%s.log' % (ROOT_DIR, container.attrs['Id']), 'a') as f:
                        f.writelines(line)
                    ws.send(dumps({
                        'type': 'eubh',
                        'project_machine_id': pivot.get('id'),
                        'data': line
                    }))
        except:
            if verbosity:
                print("Tail container error")
            ws.close()
            write_docker_log(pivot, verbosity)
    else:
        if verbosity:
            print("No container found")


class ContainerLog:
    def __init__(self, pivot, verbosity):
        write_docker_log(pivot, verbosity)



import json
import urllib3
from requests import *
from .config import Config, VERSION, EUBC_LOCAL_ADDRESS


def get_chain_sync_progress():
    return get('{0}/chain/sync-progress'.format(EUBC_LOCAL_ADDRESS)).json()


def get_eubserver_event():
    return urllib3.PoolManager().request('GET', '{0}/sse'.format(EUBC_LOCAL_ADDRESS), preload_content=False)


class Api:
    def __init__(self, key=None):
        config = Config()
        self.key = key
        self.base_url = config.get('HTTP_ADDRESS')
        self.usb_base_url = config.get('USB_HTTP_ADDRESS')
        self.headers = {'Content-Type': 'application/json', 'API-SECRET': config.get('API_SECRET')}

    def get_url(self, url):
        return '{0}/{1}'.format(self.base_url, url)

    def get_usb_base_url(self, url):
        return '%s/%s' % (self.usb_base_url, url)

    def upload_machine_and_get_task(self, data):
        return post(self.get_url('project/machine/%s' % (data.get('mac'))), data=json.dumps(data),
                    headers=self.headers).json()

    def project_machine_update_cmd(self, data):
        return post(self.get_url('project/machine/%s/update-cmd' % data.get('id')), data=data)

    def store_project_container_log(self, data):
        return post(self.get_url('project/container-log'), data=data)

    def get_project_info_by_key(self):
        return get(self.get_url('project/project/%s' % self.key), headers=self.headers).json()

    def get_project_container_lists(self, key):
        return get(self.get_url('project/container/lists/%s' % key), headers=self.headers).json()

    def get_project_container_logs_by_container_id(self, container_id):
        return get(self.get_url('project/container-log/%s/logs-by-container-id' % container_id)).json()

    def post_project_container_logs(self, data):
        return post(self.get_url('project/container-log/%s/set' % data.get('container_id')), data=data)

    def get_config_by_key(self):
        return self.get_project_info_by_key()

    def get_eubchain(self):
        return get(self.get_usb_base_url(
            '/api/base/geteubchain?version=%s&platform=%s&type=server' % (VERSION, 'linux'))).json()

    def project_store(self, data):
        return post(self.get_url('project/project'), json=data, headers=self.headers).json()

    def project_publish(self, project_id):
        return post(self.get_url('project/project/publish/{0}'.format(project_id))).json()
